module main
implicit none
integer, parameter:: dp=SELECTED_REAL_KIND(15)

contains

integer function adda(a1, a2, a3, n)

 integer, intent(in)::n
 real(dp),INTENT(IN)::a1(2,n), a2(2,n)
 real(dp),INTENT(OUT)::a3(2,n)
 a3 = 0
 a3(1,1) = a1(1,1)+a2(1,1)
 a3(2,1) = a1(2,1)+a2(2,1)
 a3(1,3) = a1(1,3)+a2(1,3)
 
 adda = 0
 End function adda

integer function bee8(bee,coord,xi,eta,det)
!
! Analytical version of the bee matrix for an 8-node quad      
!
 IMPLICIT NONE
!integer, parameter:: dp=kind(0.d0)
 real(dp),INTENT(IN)::coord(8,2),xi,eta
 real(dp),INTENT(OUT)::bee(4,16),det
 real(dp)::x1,x2,x3,x4,x5,x6,x7,x8,y1,y2,y3,y4,y5,y6,y7,y8,xi2,xi3,et2,et3,   &
   xi2et2,xi1et1,xi2et1,xi1et2,xy12,xy13,xy14,xy15,xy16,xy17,xy18,xy23,   &
   xy24,xy25,xy26,xy27,xy28,xy34,xy35,xy36,xy37,xy38,xy45,xy46,xy47,xy48, &
   xy56,xy57,xy58,xy67,xy68,xy78,xcu,xsq,xon,ecu,esq,eon,x2e2,x1e1,x2e1,  &
   x1e2,cons,bot,top,dn1dx,dn2dx,dn3dx,dn4dx,dn5dx,dn6dx,dn7dx,dn8dx,     &
   dn1dy,dn2dy,dn3dy,dn4dy,dn5dy,dn6dy,dn7dy,dn8dy,zero=0.0_dp,          &
   one=1.0_dp,two=2.0_dp,d3=3.0_dp,d4=4.0_dp,d5=5.0_dp,d6=6.0_dp,   &
   d8=8.0_dp,pt125=0.125_dp
 x1=coord(1,1)
 x2=coord(2,1)
 x3=coord(3,1)
 x4=coord(4,1)
 x5=coord(5,1)
 x6=coord(6,1)
 x7=coord(7,1)
 x8=coord(8,1)
 y1=coord(1,2)
 y2=coord(2,2)
 y3=coord(3,2)
 y4=coord(4,2)
 y5=coord(5,2)
 y6=coord(6,2)
 y7=coord(7,2)
 y8=coord(8,2)
 xi2=xi*xi
 xi3=xi2*xi
 et2=eta*eta
 et3=et2*eta
 xi2et2=xi2*et2
 xi1et1=xi*eta
 xi2et1=xi2*eta
 xi1et2=xi*et2
 xy12=x1*y2-y1*x2
 xy13=x1*y3-y1*x3
 xy14=x1*y4-y1*x4
 xy15=x1*y5-y1*x5
 xy16=x1*y6-y1*x6
 xy17=x1*y7-y1*x7
 xy18=x1*y8-y1*x8
 xy23=x2*y3-y2*x3
 xy24=x2*y4-y2*x4
 xy25=x2*y5-y2*x5
 xy26=x2*y6-y2*x6
 xy27=x2*y7-y2*x7
 xy28=x2*y8-y2*x8
 xy34=x3*y4-y3*x4
 xy35=x3*y5-y3*x5
 xy36=x3*y6-y3*x6
 xy37=x3*y7-y3*x7
 xy38=x3*y8-y3*x8
 xy45=x4*y5-y4*x5
 xy46=x4*y6-y4*x6
 xy47=x4*y7-y4*x7
 xy48=x4*y8-y4*x8
 xy56=x5*y6-y5*x6
 xy57=x5*y7-y5*x7
 xy58=x5*y8-y5*x8
 xy67=x6*y7-y6*x7
 xy68=x6*y8-y6*x8
 xy78=x7*y8-y7*x8
 xcu=-d8*xy48+d4*(-xy14+xy38+xy47+xy58)+two*(xy13+xy15-xy37-xy57)
 xsq=two*(-xy13+xy14-xy17+xy18+xy24-xy28-xy34+xy35-xy38-xy45+xy46+xy47-   &
   xy57+xy58+xy68-xy78)+one*(-xy12+xy16-xy23-xy25+xy27-xy36-xy56-xy67) 
 xon=d8*xy48+two*(xy14-xy18+xy34-xy38-xy45-xy47-xy58-xy78)+xy12-xy16+xy23-&
   xy25+xy27+xy36-xy56-xy67
 ecu=-d8*xy26+d4*(xy16+xy25+xy36+xy27)+two*(-xy15-xy17-xy35-xy37)
 esq=two*(-xy12+xy13-xy16+xy17-xy23+xy24+xy25-xy27-xy28-xy35+xy36+xy46-   &
   xy56+xy57-xy67+xy68)-xy14+xy18-xy34+xy38-xy45-xy47-xy58-xy78
 eon=d8*xy26+two*( xy12-xy16-xy23-xy25-xy27-xy36-xy56+xy67)+xy14-xy18-    &
   xy34+xy38-xy45+xy47-xy58+xy78
 x2e2=d6*(xy24-xy28+xy46+xy68)+d3*(-xy12+xy13-xy14+xy16-xy17+xy18-xy23-   &
   xy25+xy27-xy34+xy35-xy36+xy38-xy45-xy47-xy56+xy57-xy58-xy67-xy78)
 x1e1=d8*(-xy24-xy28+xy46-xy68)+d6*(-xy12+xy18+xy23+xy34-xy45-xy56+xy67+  &
   xy78)+two*(xy14-xy16+xy25+xy27-xy36+xy38-xy47+xy58)
 x2e1=d8*(xy24+xy28+xy46-xy68)+d5*( xy17-xy18-xy34+xy35-xy45+xy78)+d4*    &
   (xy12-xy16-xy23-xy25-xy27-xy36-xy56+xy67)+d3*(-xy14+xy15+xy37-xy38-    &
   xy47+xy58)
 x1e2=d8*(-xy24+xy28+xy46+xy68)+d5*(xy12-xy13+xy23+xy57-xy56-xy67)+d4*    &
   (xy14-xy18+xy34-xy38-xy45-xy47-xy58-xy78)+d3*(-xy15+xy16+xy25-xy27-    &
   xy36+xy37)
 cons= two*(-xy24+xy28-xy46-xy68)
!
 bot=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 bot=bot+xi2et2*x2e2+xi1et1*x1e1
 bot=bot+xi2et1*x2e1+xi1et2*x1e2+cons
!
 det=pt125*bot
!
 xcu=two*y3-d4*y4+two*y5
 xsq=-y2-two*y3+two*y4+y6-two*y7+two*y8
 xon=y2+two*y4 -y6-two*y8
 ecu=-two*y5+d4*y6-two*y7
 esq=-two*y2+two*y3-y4-two*y6+two*y7+y8
 eon=two*y2+y4-two*y6-y8
 x2e2=-d3*y2+d3*y3-d3*y4+d3*y6-d3*y7+d3*y8
 x1e1=-d6*y2+two*y4-two*y6+d6*y8
 x2e1=d4*y2-d3*y4+d3*y5-d4*y6+d5*y7-d5*y8
 x1e2=d5*y2-d5*y3+d4*y4-d3*y5+d3*y6-d4*y8
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn1dx=top/bot
!
 xcu=zero
 xsq=y1-y3+two*y4-y5+y7-two*y8
 xon=-y1+y3-y5+y7      
 ecu=d4*y5-d8*y6+d4*y7
 esq=two*y1-two*y3+two*y4+two*y5-two*y7-two*y8
 eon=-two*y1-two*y3-two*y5+d8*y6-two*y7      
 x2e2= d3*y1-d3*y3+d6*y4-d3*y5+d3*y7-d6*y8
 x1e1= d6*y1+d6*y3-d8*y4+two*y5+two*y7-d8*y8
 x2e1=-d4*y1-d4*y3+d8*y4-d4*y5-d4*y7+d8*y8
 x1e2=-d5*y1+d5*y3-d8*y4+d3*y5-d3*y7+d8*y8
 cons=-two*y4+two*y8
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn2dx=top/bot
!
 xcu=-two*y1-two*y7+d4*y8
 xsq=two*y1+y2-two*y4+two*y5-y6-two*y8
 xon=-y2+two*y4+y6-two*y8
 ecu=-two*y5+d4*y6-two*y7
 esq=-two*y1+two*y2-y4-two*y5+two*y6+y8
 eon=two*y2-y4-two*y6+y8
 x2e2=-d3*y1+d3*y2-d3*y4+d3*y5-d3*y6+d3*y8
 x1e1=-d6*y2+d6*y4-two*y6+two*y8
 x2e1=+d4*y2-d5*y4+d5*y5-d4*y6+d3*y7-d3*y8
 x1e2=d5*y1-d5*y2+d4*y4-d3*y6+d3*y7-d4*y8
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn3dx=top/bot
!
 xcu=d4*y1+d4*y7-d8*y8
 xsq=-two*y1-two*y2+two*y3-two*y5+two*y6+two*y7
 xon=-two*y1-two*y3-two*y5-two*y7+d8*y8
 ecu=zero
 esq=y1-two*y2+y3-y5+two*y6-y7
 eon=-y1+y3-y5+y7 
 x2e2=d3*y1-d6*y2+d3*y3-d3*y5+d6*y6-d3*y7  
 x1e1=-two*y1+d8*y2-d6*y3-d6*y5+d8*y6-two*y7 
 x2e1=d3*y1-d8*y2+d5*y3-d5*y5+d8*y6-d3*y7 
 x1e2=-d4*y1+d8*y2-d4*y3-d4*y5+d8*y6-d4*y7  
 cons=two*y2-two*y6
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn4dx=top/bot
!
 xcu=-two*y1-two*y7+d4*y8
 xsq=y2-two*y3+two*y4-y6-two*y7+two*y8
 xon=y2 +two*y4-y6-two*y8
 ecu=two*y1-d4*y2+two*y3
 esq=-two*y2+two*y3+y4-two*y6+two*y7-y8
 eon=two*y2+y4-two*y6-y8
 x2e2=d3*y2-d3*y3+d3*y4-d3*y6+d3*y7-d3*y8 
 x1e1=-two*y2+d6*y4-d6*y6+two*y8
 x2e1=-d3*y1+d4*y2-d5*y3+d5*y4-d4*y6+d3*y8
 x1e2=d3*y1-d3*y2+d4*y4-d5*y6+d5*y7-d4*y8 
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn5dx=top/bot
!
 xcu=zero     
 xsq=-y1+y3-two*y4+y5-y7+two*y8
 xon=y1-y3+y5-y7 
 ecu=-d4*y1+d8*y2-d4*y3
 esq=two*y1-two*y3-two*y4+two*y5-two*y7+two*y8
 eon=two*y1-d8*y2+two*y3+two*y5+two*y7 
 x2e2=-d3*y1+d3*y3-d6*y4+d3*y5-d3*y7+d6*y8 
 x1e1=two*y1+two*y3-d8*y4+d6*y5+d6*y7-d8*y8
 x2e1=d4*y1+d4*y3-d8*y4+d4*y5+d4*y7-d8*y8
 x1e2=-d3*y1+d3*y3-d8*y4+d5*y5-d5*y7+d8*y8 
 cons=two*y4-two*y8
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn6dx=top/bot
!
 xcu=two*y3-d4*y4+two*y5         
 xsq=two*y1-y2-two*y4+two*y5+y6-two*y8
 xon=-y2+two*y4+y6-two*y8
 ecu=two*y1-d4*y2+two*y3
 esq=-two*y1+two*y2+y4-two*y5+two*y6-y8
 eon=two*y2-y4-two*y6+y8
 x2e2=d3*y1-d3*y2+d3*y4-d3*y5+d3*y6-d3*y8 
 x1e1=-two*y2+two*y4-d6*y6+d6*y8
 x2e1=-d5*y1+d4*y2-d3*y3+d3*y4-d4*y6+d5*y8
 x1e2=d3*y2-d3*y3+d4*y4-d5*y5+d5*y6-d4*y8 
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn7dx=top/bot
!
 xcu=-d4*y3+d8*y4-d4*y5         
 xsq=-two*y1+two*y2+two*y3-two*y5-two*y6+two*y7 
 xon=two*y1+two*y3-d8*y4+two*y5+two*y7 
 ecu=zero      
 esq=-y1+two*y2-y3+y5-two*y6+y7 
 eon=y1-y3+y5-y7  
 x2e2=-d3*y1+d6*y2-d3*y3+d3*y5-d6*y6+d3*y7  
 x1e1=-d6*y1+d8*y2-two*y3-two*y5+d8*y6-d6*y7  
 x2e1=d5*y1-d8*y2+d3*y3-d3*y5+d8*y6-d5*y7  
 x1e2=d4*y1-d8*y2+d4*y3+d4*y5-d8*y6+d4*y7  
 cons=-two*y2+two*y6
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn8dx=top/bot
!
 y1=-x1
 y2=-x2
 y3=-x3
 y4=-x4
 y5=-x5
 y6=-x6
 y7=-x7
 y8=-x8
!
 xcu=two*y3-d4*y4+two*y5
 xsq=-y2-two*y3+two*y4+y6-two*y7+two*y8
 xon=y2+two*y4-y6-two*y8
 ecu=-two*y5+d4*y6-two*y7
 esq=-two*y2+two*y3-y4-two*y6+two*y7+y8
 eon=two*y2+y4-two*y6-y8
 x2e2=-d3*y2+d3*y3-d3*y4 +d3*y6-d3*y7+d3*y8
 x1e1=-d6*y2+two*y4-two*y6+d6*y8
 x2e1=d4*y2-d3*y4+d3*y5-d4*y6+d5*y7-d5*y8
 x1e2=d5*y2-d5*y3+d4*y4-d3*y5+d3*y6-d4*y8
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn1dy=top/bot
!
 xcu=zero
 xsq=y1-y3+two*y4-y5+y7-two*y8
 xon=-y1+y3-y5+y7 
 ecu=d4*y5-d8*y6+d4*y7
 esq=two*y1-two*y3+two*y4+two*y5-two*y7-two*y8
 eon=-two*y1-two*y3-two*y5+d8*y6-two*y7 
 x2e2=d3*y1-d3*y3+d6*y4-d3*y5+d3*y7-d6*y8
 x1e1=d6*y1+d6*y3-d8*y4+two*y5+two*y7-d8*y8
 x2e1=-d4*y1-d4*y3+d8*y4-d4*y5-d4*y7+d8*y8
 x1e2=-d5*y1+d5*y3-d8*y4+d3*y5-d3*y7+d8*y8
 cons=-two*y4+two*y8
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn2dy=top/bot
!
 xcu=-two*y1-two*y7+d4*y8
 xsq=two*y1+y2-two*y4+two*y5-y6-two*y8
 xon=-y2+two*y4+y6-two*y8
 ecu=-two*y5+d4*y6-two*y7
 esq=-two*y1+two*y2-y4-two*y5+two*y6+y8
 eon=two*y2-y4-two*y6+y8
 x2e2=-d3*y1+d3*y2-d3*y4+d3*y5-d3*y6+d3*y8
 x1e1=-d6*y2+d6*y4-two*y6+two*y8
 x2e1=d4*y2-d5*y4+d5*y5-d4*y6+d3*y7-d3*y8
 x1e2=d5*y1-d5*y2+d4*y4-d3*y6+d3*y7-d4*y8
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn3dy=top/bot
!
 xcu=d4*y1+d4*y7-d8*y8
 xsq=-two*y1-two*y2+two*y3-two*y5+two*y6+two*y7
 xon=-two*y1-two*y3-two*y5-two*y7+d8*y8
 ecu=zero
 esq=y1-two*y2+y3-y5+two*y6-y7
 eon=-y1+y3-y5+y7 
 x2e2=d3*y1-d6*y2+d3*y3-d3*y5+d6*y6-d3*y7  
 x1e1=-two*y1+d8*y2-d6*y3-d6*y5+d8*y6-two*y7 
 x2e1=d3*y1-d8*y2+d5*y3-d5*y5+d8*y6-d3*y7 
 x1e2=-d4*y1+d8*y2-d4*y3-d4*y5+d8*y6-d4*y7  
 cons=two*y2-two*y6
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn4dy=top/bot
!
 xcu=-two*y1-two*y7+d4*y8
 xsq=y2-two*y3+two*y4-y6-two*y7+two*y8
 xon=y2+two*y4-y6-two*y8
 ecu=two*y1-d4*y2+two*y3
 esq=-two*y2+two*y3+y4-two*y6+two*y7-y8
 eon=two*y2+y4-two*y6-y8
 x2e2=d3*y2-d3*y3+d3*y4-d3*y6+d3*y7-d3*y8 
 x1e1=-two*y2+d6*y4-d6*y6+two*y8
 x2e1=-d3*y1+d4*y2-d5*y3+d5*y4-d4*y6 +d3*y8
 x1e2=d3*y1-d3*y2+d4*y4-d5*y6+d5*y7-d4*y8 
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn5dy=top/bot
!
 xcu=zero          
 xsq=-y1+y3-two*y4+y5-y7+two*y8
 xon=y1-y3+y5-y7 
 ecu=-d4*y1+d8*y2-d4*y3
 esq=two*y1-two*y3-two*y4+two*y5-two*y7+two*y8
 eon=two*y1-d8*y2+two*y3+two*y5+two*y7 
 x2e2=-d3*y1+d3*y3-d6*y4+d3*y5-d3*y7+d6*y8 
 x1e1=two*y1+two*y3-d8*y4+d6*y5+d6*y7-d8*y8
 x2e1=d4*y1+d4*y3-d8*y4+d4*y5+d4*y7-d8*y8
 x1e2=-d3*y1+d3*y3-d8*y4+d5*y5-d5*y7+d8*y8 
 cons=two*y4-two*y8
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn6dy=top/bot
!
 xcu=two*y3-d4*y4+two*y5         
 xsq=two*y1-y2-two*y4+two*y5+y6-two*y8
 xon=-y2+two*y4+y6-two*y8
 ecu=two*y1-d4*y2+two*y3
 esq=-two*y1+two*y2+y4-two*y5+two*y6-y8
 eon=two*y2-y4-two*y6+y8
 x2e2=d3*y1-d3*y2+d3*y4-d3*y5+d3*y6-d3*y8 
 x1e1=-two*y2+two*y4-d6*y6+d6*y8
 x2e1=-d5*y1+d4*y2-d3*y3+d3*y4-d4*y6+d5*y8
 x1e2=d3*y2-d3*y3+d4*y4-d5*y5+d5*y6-d4*y8 
 cons=zero
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn7dy=top/bot
!
 xcu=-d4*y3+d8*y4-d4*y5         
 xsq=-two*y1+two*y2+two*y3-two*y5-two*y6+two*y7 
 xon=two*y1+two*y3-d8*y4+two*y5+two*y7 
 ecu=zero      
 esq=-y1+two*y2-y3+y5-two*y6+y7 
 eon=y1-y3+y5-y7  
 x2e2=-d3*y1+d6*y2-d3*y3+d3*y5-d6*y6+d3*y7  
 x1e1=-d6*y1+d8*y2-two*y3-two*y5+d8*y6-d6*y7  
 x2e1=d5*y1-d8*y2+d3*y3-d3*y5+d8*y6-d5*y7  
 x1e2=d4*y1-d8*y2+d4*y3+d4*y5-d8*y6+d4*y7  
 cons=-two*y2+two*y6
!
 top=xi3*xcu+xi2*xsq+xi*xon+et3*ecu+et2*esq+eta*eon
 top=top+xi2et2*x2e2+xi1et1*x1e1
 top=top+xi2et1*x2e1+xi1et2*x1e2+cons
!
 dn8dy=top/bot
!
 bee=zero
 bee(1,1)=dn1dx
 bee(1,3)=dn2dx
 bee(1,5)=dn3dx
 bee(1,7)=dn4dx
 bee(1,9)=dn5dx   
 bee(1,11)=dn6dx
 bee(1,13)=dn7dx   
 bee(1,15)=dn8dx
 bee(2,2)=dn1dy
 bee(2,4)=dn2dy
 bee(2,6)=dn3dy
 bee(2,8)=dn4dy
 bee(2,10)=dn5dy
 bee(2,12)=dn6dy
 bee(2,14)=dn7dy
 bee(2,16)=dn8dy
 bee(3,1)=dn1dy
 bee(3,3)=dn2dy
 bee(3,5)=dn3dy
 bee(3,7)=dn4dy
 bee(3,9)=dn5dy
 bee(3,11)=dn6dy
 bee(3,13)=dn7dy
 bee(3,15)=dn8dy
 bee(3,2)=dn1dx
 bee(3,4)=dn2dx   
 bee(3,6)=dn3dx
 bee(3,8)=dn4dx
 bee(3,10)=dn5dx
 bee(3,12)=dn6dx   
 bee(3,14)=dn7dx
 bee(3,16)=dn8dx
 bee8 = 0
END function bee8

integer function emb_2d_bc(nx1,nx2,ny1,ny2,nf)
!
! This function generates the nf array for a 2-d slope geometry.
!
 IMPLICIT NONE
 INTEGER,INTENT(IN)::nx1,nx2,ny1,ny2
 INTEGER,INTENT(OUT)::nf(:,:)
 INTEGER::nm,ic,i,j,nye
 nye=ny1+ny2
 nm=0
 ic=0
 DO i=1,2*nye
   nm=nm+1
   nf(1,nm)=0
   ic=ic+1
   nf(2,nm)=ic
 END DO
 nm=nm+1
 nf(1,nm)=0
 nf(2,nm)=0
 DO j=1,nx1-1
   DO i=1,nye
     nm=nm+1
     ic=ic+1
     nf(1,nm)=ic
     ic=ic+1
     nf(2,nm)=ic
   END DO
   nm=nm+1
   nf(1,nm)=0
   nf(2,nm)=0
   DO i=1,2*nye
     nm=nm+1
     ic=ic+1
     nf(1,nm)=ic
     ic=ic+1
     nf(2,nm)=ic
   END DO
   nm=nm+1
   nf(1,nm)=0
   nf(2,nm)=0
 END DO
   DO i=1,nye
     nm=nm+1
     ic=ic+1
     nf(1,nm)=ic
     ic=ic+1
     nf(2,nm)=ic
   END DO
   nm=nm+1
   nf(1,nm)=0
   nf(2,nm)=0
   DO i=1,2*ny1
     nm=nm+1
     ic=ic+1
     nf(1,nm)=ic
     ic=ic+1
     nf(2,nm)=ic
   END DO 
   IF(nx2==0)THEN
   DO i=1,2*ny2
     nm=nm+1
     nf(1,nm)=0
     ic=ic+1
     nf(2,nm)=ic
   END DO
   nm=nm+1
   nf(1,nm)=0
   nf(2,nm)=0
 ELSE
   DO i=1,2*ny2
     nm=nm+1
     ic=ic+1
     nf(1,nm)=ic
     ic=ic+1
     nf(2,nm)=ic
   END DO
   nm=nm+1
   nf(1,nm)=0
   nf(2,nm)=0
   DO j=1,nx2
     DO i=1,ny2
       nm=nm+1
       ic=ic+1
       nf(1,nm)=ic
       ic=ic+1
       nf(2,nm)=ic
     END DO
     nm=nm+1
     nf(1,nm)=0
     nf(2,nm)=0
     DO i=1,2*ny2
       nm=nm+1
       IF(j==nx2)THEN
         nf(1,nm)=0
       ELSE
         ic=ic+1
         nf(1,nm)=ic
       END IF
       ic=ic+1
       nf(2,nm)=ic
     END DO
     nm=nm+1
     nf(1,nm)=0
     nf(2,nm)=0
   END DO
 END IF
 emb_2d_bc = 0
RETURN
END function emb_2d_bc

integer function emb_2d_geom(iel,nx1,nx2,ny1,ny2,w1,s1,w2,h1,h2,coord,num)
!
! This function forms the nodal coordinates and numbering for a 2-d
! slope of 8-node quadrilaterals. Nodes numbering in the y-direction,
! elements numbered in the x-direction.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::w1,s1,w2,h1,h2
 INTEGER,INTENT(IN)::iel,nx1,nx2,ny1,ny2
 real(dp),INTENT(OUT)::coord(:,:)
 INTEGER,INTENT(OUT)::num(:)
 real(dp)::facx,facy,facb,facs,frh,zero=0.0_dp,pt5=0.5_dp,one=1.0_dp
 INTEGER::nxe,nye,nc,nt,ip,iq
 nxe=nx1+nx2
 nye=ny1+ny2
 nt=nx1*ny1
 nc=(3*nye+2)*nx1+2*ny1
 facx=s1/ny1
 facy=h1/ny1
 facb=(w1+s1)/nx1
 facs=zero
 IF(ny2/=0)facs=h2/ny2
 frh=zero
 IF(nx2/=0)frh=w2/nx2
 IF(iel<=nt)THEN
   iq=(iel-1)/nx1+1
   ip=iel-(iq-1)*nx1
 ELSE
   iq=(iel-nt-1)/nxe+ny1+1
   ip=iel-nt-(iq-ny1-1)*nxe
 END IF
 IF(ip<=nx1)THEN
   num(1)=(ip-1)*(3*nye+2)+2*iq+1
   num(2)=num(1)-1
   num(3)=num(1)-2
   num(4)=(ip-1)*(3*nye+2)+2*nye+iq+1
   num(5)=ip*(3*nye+2)+2*iq-1
   num(6)=num(5)+1
   num(7)=num(5)+2
   num(8)=num(4)+1
   IF(iq<=ny1)THEN
     coord(1,1)=(ip-one)*(w1+iq*facx)/nx1
     coord(3,1)=(ip-one)*(w1+(iq-1)*facx)/nx1
     coord(5,1)=ip*(w1+(iq-1)*facx)/nx1
     coord(7,1)=ip*(w1+iq*facx)/nx1
     coord(1,2)=-iq*facy
     coord(3,2)=-(iq-1)*facy
     coord(5,2)=-(iq-1)*facy
     coord(7,2)=-iq*facy
   ELSE
     coord(1,1)=(ip-one)*facb
     coord(3,1)=(ip-one)*facb
     coord(5,1)=ip*facb
     coord(7,1)=ip*facb
     coord(1,2)=-h1-(iq-ny1)*facs
     coord(3,2)=-h1-(iq-ny1-1)*facs
     coord(5,2)=-h1-(iq-ny1-1)*facs
     coord(7,2)=-h1-(iq-ny1)*facs
   END IF
 ELSE
   num(1)=nc+(ip-nx1-1)*(3*ny2+2)+2*(iq-ny1)+1
   num(2)=num(1)-1
   num(3)=num(1)-2
   num(4)=nc+(ip-nx1-1)*(3*ny2+2)+2*ny2+iq-ny1+1
   num(5)=nc+(ip-nx1)*(3*ny2+2)+2*(iq-ny1)-1
   num(6)=num(5)+1
   num(7)=num(5)+2
   num(8)=num(4)+1
   coord(1,1)=w1+s1+(ip-nx1-1)*frh
   coord(3,1)=coord(1,1)
   coord(5,1)=w1+s1+(ip-nx1)*frh
   coord(7,1)=coord(5,1)
   coord(1,2)=-h1-(iq-ny1)*facs
   coord(3,2)=-h1-(iq-ny1-1)*facs
   coord(5,2)=-h1-(iq-ny1-1)*facs
   coord(7,2)=-h1-(iq-ny1)*facs
 END IF
 coord(2:6:2,:)=pt5*(coord(1:5:2,:)+coord(3:7:2,:))
 coord(8,:)=pt5*(coord(7,:)+coord(1,:))
RETURN
END function emb_2d_geom
!
integer function num_to_g(num,nf,g)
!
! This function finds the g vector from num and nf.
!
 IMPLICIT NONE
 INTEGER,INTENT(IN)::num(:),nf(:,:)  
 INTEGER,INTENT(OUT)::g(:)
 INTEGER::i,k,nod,nodof 
 nod=UBOUND(num,1) 
 nodof=UBOUND(nf,1)
 DO i=1,nod
   k=i*nodof
   g(k-nodof+1:k)=nf(:,num(i))
 END DO
RETURN
END function num_to_g   

integer function fkdiag(kdiag,g)
!
! This function computes the skyline profile.
!
 IMPLICIT NONE
 INTEGER,INTENT(IN)::g(:)
 INTEGER,INTENT(OUT)::kdiag(:)
 INTEGER::idof,i,iwp1,j,im,k
 idof=SIZE(g)
 DO i=1,idof
   iwp1=1
   IF(g(i)/=0)THEN
     DO j=1,idof
       IF(g(j)/=0)THEN
         im=g(i)-g(j)+1
         IF(im>iwp1)iwp1=im
       END IF
     END DO
     k=g(i)
     IF(iwp1>kdiag(k))kdiag(k)=iwp1
   END IF
 END DO
RETURN
END function fkdiag
!
integer function sample(element,s,wt)
!
! This function returns the local coordinates and weighting coefficients
! of the integrating points.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(OUT)::s(:,:)
 real(dp),INTENT(OUT),OPTIONAL::wt(:)
 CHARACTER(*),INTENT(IN)::element
 INTEGER::nip
 real(dp)::root3,r15,w(3),v(9),b,c
 root3=1.0_dp/SQRT(3.0_dp)
 r15=0.2_dp*SQRT(15.0_dp)
 nip=UBOUND(s,1)
 w=(/5.0/9.0,8.0/9.0,5.0/9.0/)
 v=(/5.0/9.0*w,8.0/9.0*w,5.0/9.0*w/)
 SELECT CASE(element)
 CASE('line')
   SELECT CASE(nip)
   CASE(1)
     s(1,1)=0.0_dp
     wt(1) =2.0_dp
   CASE(2)
     s(1,1)=-0.577350269189626
     s(2,1)= 0.577350269189626
     wt(1) = 1.000000000000000
     wt(2) = 1.000000000000000
   CASE(3)
     s(1,1)=-0.774596669241484
     s(2,1)= 0.000000000000000
     s(3,1)= 0.774596669241484
     wt(1) = 0.555555555555556
     wt(2) = 0.888888888888889
     wt(3) = 0.555555555555556
   CASE(4)
     s(1,1)=-0.861136311594053
     s(2,1)=-0.339981043584856
     s(3,1)= 0.339981043584856
     s(4,1)= 0.861136311594053
     wt(1) = 0.347854845137454
     wt(2) = 0.652145154862546
     wt(3) = 0.652145154862546
     wt(4) = 0.347854845137454
   CASE(5)
     s(1,1)=-0.906179845938664
     s(2,1)=-0.538469310105683
     s(3,1)= 0.000000000000000
     s(4,1)= 0.538469310105683
     s(5,1)= 0.906179845938664
     wt(1) = 0.236926885056189
     wt(2) = 0.478628670499366
     wt(3) = 0.568888888888889
     wt(4) = 0.478628670499366
     wt(5) = 0.236926885056189
   CASE(6)
     s(1,1)=-0.932469514203152
     s(2,1)=-0.661209386466265
     s(3,1)=-0.238619186083197
     s(4,1)= 0.238619186083197
     s(5,1)= 0.661209386466265
     s(6,1)= 0.932469514203152
     wt(1) = 0.171324492379170
     wt(2) = 0.360761573048139
     wt(3) = 0.467913934572691
     wt(4) = 0.467913934572691
     wt(5) = 0.360761573048139
     wt(6) = 0.171324492379170
   CASE(7)
     s(1,1)=-0.9491079123427585245261897
     s(2,1)=-0.7415311855993944398638648
     s(3,1)=-0.4058451513773971669066064
     s(4,1)= 0.000000000000000
     s(5,1)= 0.4058451513773971669066064
     s(6,1)= 0.7415311855993944398638648
     s(7,1)= 0.9491079123427585245261897
     wt(1) = 0.1294849661688696932706114
     wt(2) = 0.2797053914892766679014678
     wt(3) = 0.3818300505051189449503698
     wt(4) = 0.4179591836734693877551020
     wt(5) = 0.3818300505051189449503698
     wt(6) = 0.2797053914892766679014678
     wt(7) = 0.1294849661688696932706114
   CASE(8)
     s(1,1)=-0.9602898564975362316835609
     s(2,1)=-0.7966664774136267395915539
     s(3,1)=-0.5255324099163289858177390
     s(4,1)=-0.1834346424956498049394761
     s(5,1)= 0.1834346424956498049394761
     s(6,1)= 0.5255324099163289858177390
     s(7,1)= 0.7966664774136267395915539
     s(8,1)= 0.9602898564975362316835609
     wt(1) = 0.1012285362903762591525314
     wt(2) = 0.2223810344533744705443560
     wt(3) = 0.3137066458778872873379622
     wt(4) = 0.3626837833783619829651504
     wt(5) = 0.3626837833783619829651504
     wt(6) = 0.3137066458778872873379622
     wt(7) = 0.2223810344533744705443560
     wt(8) = 0.1012285362903762591525314
   CASE(9)
     s(1,1)=-0.9681602395076260898355762
     s(2,1)=-0.8360311073266357942994298    
     s(3,1)=-0.6133714327005903973087020
     s(4,1)=-0.3242534234038089290385380    
     s(5,1)= 0.000000000000000                            
     s(6,1)= 0.3242534234038089290385380                            
     s(7,1)= 0.6133714327005903973087020                            
     s(8,1)= 0.8360311073266357942994298                            
     s(9,1)= 0.9681602395076260898355762                            
     wt(1) = 0.0812743883615744119718922                            
     wt(2) = 0.1806481606948574040584720                            
     wt(3) = 0.2606106964029354623187429                            
     wt(4) = 0.3123470770400028400686304                            
     wt(5) = 0.3302393550012597631645251                            
     wt(6) = 0.3123470770400028400686304                            
     wt(7) = 0.2606106964029354623187429                            
     wt(8) = 0.1806481606948574040584720                            
     wt(9) = 0.0812743883615744119718922                            
   CASE(10)
     s(1,1)=-0.9739065285171717200779640            
     s(2,1)=-0.8650633666889845107320967 
     s(3,1)=-0.6794095682990244062343274 
     s(4,1)=-0.4333953941292471907992659 
     s(5,1)=-0.1488743389816312108848260 
     s(6,1)= 0.1488743389816312108848260 
     s(7,1)= 0.4333953941292471907992659 
     s(8,1)= 0.6794095682990244062343274 
     s(9,1)= 0.8650633666889845107320967 
    s(10,1)= 0.9739065285171717200779640 
     wt(1) = 0.0666713443086881375935688                     
     wt(2) = 0.1494513491505805931457763                     
     wt(3) = 0.2190863625159820439955349                     
     wt(4) = 0.2692667193099963550912269                     
     wt(5) = 0.2955242247147528701738930                     
     wt(6) = 0.2955242247147528701738930                      
     wt(7) = 0.2692667193099963550912269                     
     wt(8) = 0.2190863625159820439955349                     
     wt(9) = 0.1494513491505805931457763                     
    wt(10) = 0.0666713443086881375935688                     
   CASE DEFAULT                              
     WRITE(*,*)"Wrong number of integrating points for a line"
   END SELECT
 CASE('triangle')
 SELECT CASE(nip)
   CASE(1)
     s(1,1)= 0.333333333333333
     s(1,2)= 0.333333333333333
     wt(1) = 0.500000000000000
   CASE(3)
     s(1,1)= 0.500000000000000
     s(1,2)= 0.500000000000000
     s(2,1)= 0.500000000000000
     s(2,2)= 0.000000000000000
     s(3,1)= 0.000000000000000
     s(3,2)= 0.500000000000000
     wt(1:3)=0.333333333333333
     wt=0.5_dp*wt
   CASE(4)
     s(1,1)= 0.6
     s(1,2)= 0.2
     s(2,1)= 0.2
     s(2,2)= 0.6
     s(3,1)= 0.2
     s(3,2)= 0.2
     s(4,1)= 0.333333333333333
     s(4,2)= 0.333333333333333
     wt(1:3)= 0.520833333333333
     wt(4)=  -0.5625
     wt=0.5_dp*wt
   CASE(6)
     s(1,1)= 0.816847572980459
     s(1,2)= 0.091576213509771
     s(2,1)= 0.091576213509771
     s(2,2)= 0.816847572980459
     s(3,1)= 0.091576213509771
     s(3,2)= 0.091576213509771
     s(4,1)= 0.108103018168070
     s(4,2)= 0.445948490915965
     s(5,1)= 0.445948490915965
     s(5,2)= 0.108103018168070
     s(6,1)= 0.445948490915965
     s(6,2)= 0.445948490915965
     wt(1:3)=0.109951743655322
     wt(4:6)=0.223381589678011
     wt=0.5_dp*wt
   CASE(7)
     s(1,1)= 0.333333333333333
     s(1,2)= 0.333333333333333
     s(2,1)= 0.797426985353087
     s(2,2)= 0.101286507323456
     s(3,1)= 0.101286507323456
     s(3,2)= 0.797426985353087
     s(4,1)= 0.101286507323456
     s(4,2)= 0.101286507323456
     s(5,1)= 0.470142064105115
     s(5,2)= 0.059715871789770
     s(6,1)= 0.059715871789770
     s(6,2)= 0.470142064105115
     s(7,1)= 0.470142064105115
     s(7,2)= 0.470142064105115
     wt(1) = 0.225000000000000
     wt(2:4)=0.125939180544827
     wt(5:7)=0.132394152788506
     wt=0.5_dp*wt
   CASE(12)
     s(1,1)= 0.873821971016996
     s(1,2)= 0.063089014491502
     s(2,1)= 0.063089014491502
     s(2,2)= 0.873821971016996
     s(3,1)= 0.063089014491502
     s(3,2)= 0.063089014491502
     s(4,1)= 0.501426509658179
     s(4,2)= 0.249286745170910
     s(5,1)= 0.249286745170910
     s(5,2)= 0.501426509658179
     s(6,1)= 0.249286745170910
     s(6,2)= 0.249286745170910
     s(7,1) =0.053145049844817
     s(7,2) =0.310352451033784
     s(8,1) =0.310352451033784
     s(8,2) =0.053145049844817
     s(9,1) =0.053145049844817
     s(9,2) =0.636502499121398
     s(10,1)=0.310352451033784
     s(10,2)=0.636502499121398
     s(11,1)=0.636502499121398
     s(11,2)=0.053145049844817
     s(12,1)=0.636502499121398
     s(12,2)=0.310352451033784
     wt(1:3)=0.050844906370207
     wt(4:6)=0.116786275726379
     wt(7:12)=0.082851075618374
     wt=0.5_dp*wt
   CASE(16)
     s(1,1)=0.333333333333333
     s(1,2)=0.333333333333333
     s(2,1)=0.658861384496478
     s(2,2)=0.170569307751761
     s(3,1)=0.170569307751761
     s(3,2)=0.658861384496478
     s(4,1)=0.170569307751761
     s(4,2)=0.170569307751761
     s(5,1)=0.898905543365938
     s(5,2)=0.050547228317031
     s(6,1)=0.050547228317031
     s(6,2)=0.898905543365938
     s(7,1)=0.050547228317031
     s(7,2)=0.050547228317031
     s(8,1)=0.081414823414554
     s(8,2)=0.459292588292723
     s(9,1)=0.459292588292723
     s(9,2)=0.081414823414554
     s(10,1)=0.459292588292723
     s(10,2)=0.459292588292723
     s(11,1)=0.008394777409958
     s(11,2)=0.263112829634638
     s(12,1)=0.008394777409958
     s(12,2)=0.728492392955404
     s(13,1)=0.263112829634638
     s(13,2)=0.008394777409958
     s(14,1)=0.263112829634638
     s(14,2)=0.728492392955404
     s(15,1)=0.728492392955404
     s(15,2)=0.008394777409958
     s(16,1)=0.728492392955404
     s(16,2)=0.263112829634638
     wt(1)=0.144315607677787
     wt(2:4)=0.103217370534718
     wt(5:7)=0.032458497623198
     wt(8:10)=0.095091634267284
     wt(11:16)=0.027230314174435
     wt=0.5_dp*wt
   CASE DEFAULT
     WRITE(*,*)"wrong number of integrating points for a triangle"
   END SELECT
 CASE('quadrilateral')
   SELECT CASE(nip)
   CASE(1)
     s(1,1)=0.0_dp
     s(1,2)=0.0_dp
     wt(1)=4.0
   CASE(4)
     s(1,1)=-root3
     s(1,2)= root3
     s(2,1)= root3
     s(2,2)= root3
     s(3,1)=-root3
     s(3,2)=-root3
     s(4,1)= root3
     s(4,2)=-root3
     wt=1.0_dp
   CASE(9)
     s(1:7:3,1)=-r15
     s(2:8:3,1)=0.0_dp
     s(3:9:3,1)=r15
     s(1:3,2)  =r15
     s(4:6,2)  =0.0_dp
     s(7:9,2)  =-r15
     wt= v
   CASE(16)
     s(1:13:4,1)=-0.861136311594053
     s(2:14:4,1)=-0.339981043584856
     s(3:15:4,1)= 0.339981043584856
     s(4:16:4,1)= 0.861136311594053
     s(1:4,2)   = 0.861136311594053
     s(5:8,2)   = 0.339981043584856
     s(9:12,2)  =-0.339981043584856
     s(13:16,2) =-0.861136311594053
     wt(1)      = 0.121002993285602
     wt(4)      = wt(1)
     wt(13)     = wt(1)
     wt(16)     = wt(1)
     wt(2)      = 0.226851851851852
     wt(3)      = wt(2)
     wt(5)      = wt(2)
     wt(8)      = wt(2)
     wt(9)      = wt(2)
     wt(12)     = wt(2)
     wt(14)     = wt(2)
     wt(15)     = wt(2)
     wt(6)      = 0.425293303010694
     wt(7)      = wt(6)
     wt(10)     = wt(6)
     wt(11)     = wt(6)
   CASE(25)
     s(1:21:5,1)= 0.906179845938664
     s(2:22:5,1)= 0.538469310105683
     s(3:23:5,1)= 0.0_dp
     s(4:24:5,1)=-0.538469310105683
     s(5:25:5,1)=-0.906179845938664
     s( 1: 5,2) = 0.906179845938664
     s( 6:10,2) = 0.538469310105683
     s(11:15,2) = 0.0_dp
     s(16:20,2) =-0.538469310105683
     s(21:25,2) =-0.906179845938664
     wt(1) =0.056134348862429
     wt(2) =0.113400000000000
     wt(3) =0.134785072387521
     wt(4) =0.113400000000000
     wt(5) =0.056134348862429
     wt(6) =0.113400000000000
     wt(7) =0.229085404223991
     wt(8) =0.272286532550750
     wt(9) =0.229085404223991
     wt(10)=0.113400000000000
     wt(11)=0.134785072387521
     wt(12)=0.272286532550750
     wt(13)=0.323634567901235
     wt(14)=0.272286532550750
     wt(15)=0.134785072387521
     wt(16)=0.113400000000000
     wt(17)=0.229085404223991
     wt(18)=0.272286532550750
     wt(19)=0.229085404223991
     wt(20)=0.113400000000000
     wt(21)=0.056134348862429
     wt(22)=0.113400000000000
     wt(23)=0.134785072387521
     wt(24)=0.113400000000000
     wt(25)=0.056134348862429
   CASE DEFAULT
     WRITE(*,*)"wrong number of integrating points for a quadrilateral"
   END SELECT
 CASE('tetrahedron')
!                       for tetrahedra weights multiplied by 1/6
   SELECT CASE(nip)
   CASE(1)
     s(1,1)=0.25
     s(1,2)=0.25
     s(1,3)=0.25
     wt(1)=1.0_dp/6.0
   CASE(4)
     s(1,1)=0.58541020
     s(1,2)=0.13819660
     s(1,3)=s(1,2)
     s(2,2)=s(1,1)
     s(2,3)=s(1,2)
     s(2,1)=s(1,2)
     s(3,3)=s(1,1)
     s(3,1)=s(1,2)
     s(3,2)=s(1,2)
     s(4,1)=s(1,2)
     s(4,2)=s(1,2)
     s(4,3)=s(1,2)
     wt(1:4)=0.25/6.0
   CASE(5)
     s(1,1)=0.25
     s(1,2)=0.25
     s(1,3)=0.25
     s(2,1)=0.5_dp
     s(2,2)=1.0_dp/6.0
     s(2,3)=s(2,2)
     s(3,2)=0.5_dp
     s(3,3)=1.0_dp/6.0
     s(3,1)=s(3,3)
     s(4,3)=0.5_dp
     s(4,1)=1.0_dp/6.0
     s(4,2)=s(4,1)
     s(5,1)=1.0_dp/6.0
     s(5,2)=s(5,1)
     s(5,3)=s(5,1)
     wt(1)=-0.8
     wt(2)=9.0/20.0
     wt(3:5)=wt(2)
     wt=wt/6.0
   CASE DEFAULT
     WRITE(*,*)"wrong number of integrating points for a tetrahedron"
   END SELECT
 CASE('hexahedron')
   SELECT CASE(nip)
   CASE(1)
     s(1,1:3)=0.0_dp
     wt(1)=8.0
   CASE(8)
     s(1,1)= root3
     s(1,2)= root3
     s(1,3)= root3
     s(2,1)= root3
     s(2,2)= root3
     s(2,3)=-root3
     s(3,1)= root3
     s(3,2)=-root3
     s(3,3)= root3
     s(4,1)= root3
     s(4,2)=-root3
     s(4,3)=-root3
     s(5,1)=-root3
     s(5,2)= root3
     s(5,3)= root3
     s(6,1)=-root3
     s(6,2)=-root3
     s(6,3)= root3
     s(7,1)=-root3
     s(7,2)= root3
     s(7,3)=-root3
     s(8,1)=-root3
     s(8,2)=-root3
     s(8,3)=-root3
     wt=1.0_dp
   CASE(14)
     b=0.795822426
     c=0.758786911
     wt(1:6)=0.886426593
     wt(7:14)=0.335180055
     s(1,1)=-b
     s(2,1)=b
     s(3,2)=-b
     s(4,2)=b
     s(5,3)=-b
     s(6,3)=b
     s(7:,:)=c
     s(7,1)=-c
     s(7,2)=-c
     s(7,3)=-c
     s(8,2)=-c
     s(8,3)=-c
     s(9,1)=-c
     s(9,3)=-c
     s(10,3)=-c
     s(11,1)=-c
     s(11,2)=-c
     s(12,2)=-c
     s(13,1)=-c
   CASE(15)
     b=1.0_dp
     c       =0.674199862
     wt(1)   =1.564444444
     wt(2:7) =0.355555556
     wt(8:15)=0.537777778
     s(2,1)=-b
     s(3,1)=b
     s(4,2)=-b
     s(5,2)=b
     s(6,3)=-b
     s(7,3)=b
     s(8:,:)=c
     s(8,1)=-c
     s(8,2)=-c
     s(8,3)=-c
     s(9,2)=-c
     s(9,3)=-c
     s(10,1)=-c
     s(10,3)=-c
     s(11,3)=-c
     s(12,1)=-c
     s(12,2)=-c
     s(13,2)=-c
     s(14,1)=-c
   CASE(27)
     wt=(/5.0/9.0*v,8.0/9.0*v,5.0/9.0*v/)
     s(1:7:3,1)=-r15
     s(2:8:3,1)=0.0_dp
     s(3:9:3,1)=r15
     s(1:3,3)=r15
     s(4:6,3)=0.0_dp
     s(7:9,3)=-r15
     s(1:9,2)=-r15
     s(10:16:3,1)=-r15
     s(11:17:3,1)=0.0_dp
     s(12:18:3,1)=r15
     s(10:12,3)=r15
     s(13:15,3)=0.0_dp
     s(16:18,3)=-r15
     s(10:18,2)=0.0_dp
     s(19:25:3,1)=-r15
     s(20:26:3,1)=0.0_dp
     s(21:27:3,1)=r15
     s(19:21,3)= r15
     s(22:24,3)=0.0_dp
     s(25:27,3)=-r15
     s(19:27,2)= r15
   CASE DEFAULT
     WRITE(*,*)"wrong number of integrating points for a hexahedron"
   END SELECT
 CASE DEFAULT
   WRITE(*,*)"not a valid element type"
 END SELECT
RETURN
END function sample

integer function deemat(dee,e,v)
!
! This function returns the elastic dee matrix for ih=3 (plane strain),
! ih=4 (axisymmetry or plane strain elastoplasticity) or ih=6
! (three dimensions).
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::e,v
 real(dp),INTENT(OUT)::dee(:,:)
 real(dp)::v1,v2,c,vv,zero=0.0_dp,pt5=0.5_dp,one=1.0_dp,two=2.0_dp
 INTEGER::i,ih
 dee=zero  
 ih=UBOUND(dee,1)
 v1=one-v
 c=e/((one+v)*(one-two*v))
 SELECT CASE(ih)
 CASE(3)
   dee(1,1)=v1*c
   dee(2,2)=v1*c
   dee(1,2)=v*c
   dee(2,1)=v*c
   dee(3,3)=pt5*c*(one-two*v)
 CASE(4)
   dee(1,1)=v1*c
   dee(2,2)=v1*c
   dee(4,4)=v1*c
   dee(3,3)=pt5*c*(one-two*v) 
   dee(1,2)=v*c
   dee(2,1)=v*c
   dee(1,4)=v*c
   dee(4,1)=v*c
   dee(2,4)=v*c
   dee(4,2)=v*c
 CASE(6)
   v2=v/(one-v)
   vv=(one-two*v)/(one-v)*pt5
   DO i=1,3
     dee(i,i)=one
   END DO
   DO i=4,6
     dee(i,i)=vv
   END DO
   dee(1,2)=v2
   dee(2,1)=v2
   dee(1,3)=v2
   dee(3,1)=v2
   dee(2,3)=v2
   dee(3,2)=v2
   dee=dee*e/(two*(one+v)*vv)
 CASE DEFAULT
   WRITE(*,*)'wrong size for dee matrix'
 END SELECT
RETURN
END function deemat    
!
integer function shape_fun(fun,points,i)
!
!   This function computes the values of the shape functions.
!   to local coordinates
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 INTEGER,INTENT(in)::i
 real(dp),INTENT(IN)::points(:,:)
 real(dp),INTENT(OUT)::fun(:)
 real(dp)::eta,xi,etam,etap,xim,xip,zetam,zetap,c1,c2,c3     
 real(dp)::t1,t2,t3,t4,t5,t6,t7,t8,t9
 real(dp)::zeta,xi0,eta0,zeta0
 INTEGER::xii(20),etai(20),zetai(20),l,ndim,nod
 REAL,PARAMETER::pt125=0.125_dp,pt25=0.25_dp,pt5=0.5_dp,pt75=0.75_dp, &
   one=1.0_dp,two=2.0_dp,d3=3.0_dp,d4=4.0_dp,d8=8.0_dp,d9=9.0_dp,   &
   d16=16.0_dp,d27=27.0_dp,d32=32.0_dp,d64=64.0_dp,d128=128.0_dp
 ndim=UBOUND(points,2)
 nod=UBOUND(fun,1)  
 SELECT CASE(ndim)
 CASE(1) ! one dimensional case
   xi=points(i,1)
   SELECT CASE(nod)
   CASE(2)
     t1=-one-xi 
     t2= one-xi
     fun(1)=t2/two 
     fun(2)=-t1/two
   CASE(3)
     t1=-one-xi 
     t2=-xi 
     t3=one-xi
     fun(1)=t2*t3/two 
     fun(2)=-t1*t3 
     fun(3)=t1*t2/two
   CASE(4)
     t1=-one-xi 
     t2=-one/d3-xi 
     t3=one/d3-xi 
     t4=one-xi
     fun(1)=t2*t3*t4*d9/d16  
     fun(2)=-t1*t3*t4*d27/d16
     fun(3)=t1*t2*t4*d27/d16 
     fun(4)=-t1*t2*t3*d9/d16
   CASE(5)
     t1=-one -xi 
     t2=-pt5-xi 
     t3=-xi 
     t4=pt5-xi 
     t5=one-xi
     fun(1)=t2*t3*t4*t5*two/d3 
     fun(2)=-t1*t3*t4*t5*d8/d3
     fun(3)=t1*t2*t4*t5*d4 
     fun(4)=-t1*t2*t3*t5*d8/d3
     fun(5)=t1*t2*t3*t4*two/d3
   CASE DEFAULT
     WRITE(*,*)"wrong number of nodes in shape_fun"
   END SELECT
 CASE(2) ! two dimensional case
   c1=points(i,1)
   c2=points(i,2)
   c3=one-c1-c2 
   xi=points(i,1)
   eta=points(i,2)
   etam=pt25*(one-eta)
   etap=pt25*(one+eta)
   xim=pt25*(one-xi)
   xip=pt25*(one+xi)
   SELECT CASE(nod)
   CASE(3)
     fun = (/c1,c3,c2/)  
   CASE(6)
     fun(1)=(two*c1-one)*c1 
     fun(2)=d4*c3*c1
     fun(3)=(two*c3-one)*c3 
     fun(4)=d4*c2*c3      
     fun(5)=(two*c2-one)*c2
     fun(6)=d4*c1*c2 
   CASE(10)
     fun(1)= ((d3*c1-one)*(d3*c1-two)*c1)/two
     fun(2)= -(d9*(d3*c1-one)*(c1+c2-one)*c1)/two
     fun(3)=  (d9*(d3*c1+d3*c2-two)*(c1+c2-one)*c1)/two
     fun(4)=-((d3*c1+d3*c2-one)*(d3*c1+d3*c2-two)*(c1+c2-one))/two    
     fun(5)=  (d9*(d3*c1+d3*c2-two)*(c1+c2-one)*c2)/two
     fun(6)= -(d9*(c1+c2-one)*(d3*c2-one)*c2)/two
     fun(7)= ((d3*c2-one)*(d3*c2-two)*c2)/two
     fun(8)=  (d9*(d3*c2-one)*c1*c2)/two
     fun(9)=  (d9*(d3*c1-one)*c1*c2)/two
     fun(10)=-d27*((c2-one)+c1)*c1*c2
   CASE(15)
     t1=c1-pt25  
     t2=c1-pt5 
     t3=c1-pt75   
     t4=c2-pt25
     t5=c2-pt5   
     t6=c2-pt75 
     t7=c3-pt25  
     t8=c3-pt5 
     t9=c3-pt75
     fun(1)=d32/d3*c1*t1*t2*t3   
     fun(2)=d128/d3*c3*c1*t1*t2
     fun(3)=d64*c3*c1*t1*t7      
     fun(4)=d128/d3*c3*c1*t7*t8
     fun(5)=d32/d3*c3*t7*t8*t9   
     fun(6)=d128/d3*c2*c3*t7*t8
     fun(7)=d64*c2*c3*t4*t7      
     fun(8)=d128/d3*c2*c3*t4*t5
     fun(9)=d32/d3*c2*t4*t5*t6   
     fun(10)=d128/d3*c1*c2*t4*t5
     fun(11)=d64*c1*c2*t1*t4     
     fun(12)=d128/d3*c1*c2*t1*t2
     fun(13)=d128*c1*c2*t1*c3    
     fun(15)=d128*c1*c2*c3*t4
     fun(14)=d128*c1*c2*c3*t7      
   CASE(4)
     fun=(/d4*xim*etam,d4*xim*etap,d4*xip*etap,d4*xip*etam/)
   CASE(5)
     fun=(/d4*xim*etam-pt25*(one-xi**2)*(one-eta**2),			  &
           d4*xim*etap-pt25*(one-xi**2)*(one-eta**2),			  &
           d4*xip*etap-pt25*(one-xi**2)*(one-eta**2),			  &
           d4*xip*etam-pt25*(one-xi**2)*(one-eta**2),			  &
           (one-xi**2)*(one-eta**2)/)
   CASE(8)
     fun=(/d4*etam*xim*(-xi-eta-one),d32*etam*xim*etap,                   &
           d4*etap*xim*(-xi+eta-one),d32*xim*xip*etap,                    &
           d4*etap*xip*(xi+eta-one), d32*etap*xip*etam,                   &
           d4*xip*etam*(xi-eta-one), d32*xim*xip*etam/)
   CASE(9)
     etam=eta-one
     etap=eta+one
     xim=xi-one
     xip=xi+one
     fun=(/pt25*xi*xim*eta*etam,-pt5*xi*xim*etap*etam,                    &
           pt25*xi*xim*eta*etap,-pt5*xip*xim*eta*etap,                    &
           pt25*xi*xip*eta*etap,-pt5*xi*xip*etap*etam,                    &
           pt25*xi*xip*eta*etam,-pt5*xip*xim*eta*etam,                    &
           xip*xim*etap*etam/)
   CASE DEFAULT
     WRITE(*,*)"wrong number of nodes in shape_fun"
   END SELECT
 CASE(3) ! d3 dimensional case
   xi=points(i,1)
   eta=points(i,2)
   zeta=points(i,3)
   etam=one-eta 
   xim=one-xi  
   zetam=one-zeta
   etap=eta+one 
   xip=xi+one   
   zetap=zeta+one
   SELECT CASE(nod)
   CASE(4)
     fun(1)=xi   
     fun(2)=eta 
     fun(3)=zeta 
     fun(4)=one-fun(1)-fun(2)-fun(3)
   CASE(8)
     fun=(/pt125*xim*etam*zetam,pt125*xim*etam*zetap,                     &
           pt125*xip*etam*zetap,pt125*xip*etam*zetam,                     &
           pt125*xim*etap*zetam,pt125*xim*etap*zetap,                     &
           pt125*xip*etap*zetap,pt125*xip*etap*zetam/)
   CASE(14) ! type 6 element
     fun(1) = (xi*eta+xi*zeta+two*xi+eta*zeta+two*eta+two*zeta+two)*      &
       (xi-one)*(eta-one)*(zeta-one)/d8
     fun(2) =-(xi*eta-xi*zeta+two*xi-eta*zeta+two*eta-two*zeta+two)*      &
       (xi-one)*(eta-one)*(zeta+one)/d8
     fun(3) =-(xi*eta-xi*zeta+two*xi+eta*zeta-two*eta+two*zeta-two)*      &
       (xi+one)*(eta-one)*(zeta+one)/d8
     fun(4) = (xi*eta+xi*zeta+two*xi-eta*zeta-two*eta-two*zeta-two)*      &
       (xi+one)*(eta-one)*(zeta-one)/d8
     fun(5) =-(xi+one)*(xi-one)*(eta-one)*(zeta+one)*(zeta-one)/two
     fun(6) =-(xi-one)*(eta+one)*(eta-one)*(zeta+one)*(zeta-one)/two
     fun(7) = (xi+one)*(xi-one)*(eta+one)*(eta-one)*(zeta+one)/two
     fun(8) = (xi+one)*(eta+one)*(eta-one)*(zeta+one)*(zeta-one)/two
     fun(9) =-(xi+one)*(xi-one)*(eta+one)*(eta-one)*(zeta-one)/two
     fun(10)= (xi*eta-xi*zeta-two*xi+eta*zeta+two*eta-two*zeta-two)*      &
       (xi-one)*(eta+one)*(zeta-one)/d8
     fun(11)=-(xi*eta+xi*zeta-two*xi-eta*zeta+two*eta+two*zeta-two)*      &
       (xi-one)*(eta+one)*(zeta+one)/d8
     fun(12)=-(xi*eta+xi*zeta-two*xi+eta*zeta-two*eta-two*zeta+two)*      &
       (xi+one)*(eta+one)*(zeta+one)/d8
     fun(13)= (xi*eta-xi*zeta-two*xi-eta*zeta-two*eta+two*zeta+two)*      &
       (xi+one)*(eta+one)*(zeta-one)/d8
     fun(14)= (xi+one)*(xi-one)*(eta+one)*(zeta+one)*(zeta-one)/two
   CASE(20)
     xii=(/-1,-1,-1,0,1,1,1,0,-1,-1,1,1,-1,-1,-1,0,1,1,1,0/)
     etai=(/-1,-1,-1,-1,-1,-1,-1,-1,0,0,0,0,1,1,1,1,1,1,1,1/)
     zetai=(/-1,0,1,1,1,0,-1,-1,-1,1,1,-1,-1,0,1,1,1,0,-1,-1/)
     DO l=1,20
       xi0=xi*xii(l)
       eta0=eta*etai(l)
       zeta0=zeta*zetai(l)
       IF(l==4.OR.l==8.OR.l==16.OR.l==20)THEN
         fun(l)=pt25*(one-xi*xi)*(one+eta0)*(one+zeta0)
       ELSE IF(l>=9.AND.l<=12)THEN
         fun(l)=pt25*(one+xi0)*(one-eta*eta)*(one+zeta0)
       ELSE IF(l==2.OR.l==6.OR.l==14.OR.l==18)THEN
         fun(l)=pt25*(one+xi0)*(one+eta0)*(one-zeta*zeta)
       ELSE
         fun(l)=pt125*(one+xi0)*(one+eta0)*(one+zeta0)*(xi0+eta0+zeta0-2)
       END IF
     END DO
   CASE DEFAULT
     WRITE(*,*)"wrong number of nodes in shape_fun"
   END SELECT
 CASE DEFAULT
   WRITE(*,*)"wrong number of dimensions in shape_fun"
 END SELECT
RETURN
END function shape_fun 
!
integer function fsparv(kv,km,g,kdiag)
!
! This function assembles element matrices into a symmetric skyline
! global matrix.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 INTEGER,INTENT(IN)::g(:),kdiag(:)
 real(dp),INTENT(IN)::km(:,:)
 real(dp),INTENT(OUT)::kv(:) 
 INTEGER::i,idof,k,j,iw,ival
 idof=UBOUND(g,1)
 DO i=1,idof
   k=g(i)
   IF(k/=0)THEN
     DO j=1,idof
       IF(g(j)/=0)THEN
         iw=k-g(j)
         IF(iw>=0)THEN
           ival=kdiag(k)-iw
           kv(ival)=kv(ival)+km(i,j) 
         END IF
       END IF
     END DO
   END IF
 END DO
RETURN
END function fsparv
!
integer function sparin(kv,kdiag)
!
! This function performs Cholesky factorisation on a symmetric
! skyline global matrix.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN OUT)::kv(:)
 INTEGER,INTENT(IN)::kdiag(:)
 INTEGER::n,i,ki,l,kj,j,ll,m,k
 real(dp)::x
 n=UBOUND(kdiag,1)  
 kv(1)=SQRT(kv(1))
 DO i=2,n
   ki=kdiag(i)-i
   l=kdiag(i-1)-ki+1
   DO j=l,i
     x=kv(ki+j)
     kj=kdiag(j)-j
     IF(j/=1)THEN
       ll=kdiag(j-1)-kj+1
       ll=max(l,ll)
       IF(ll/=j)THEN
         m=j-1
         DO k=ll,m 
           x=x-kv(ki+k)*kv(kj+k) 
         END DO
       END IF
     END IF
     kv(ki+j)=x/kv(kj+j)
   END DO
   kv(ki+i)=SQRT(x)
 END DO
RETURN
END function sparin
!
integer function spabac(kv,loads,kdiag)
!
! This function performs Cholesky forward and back-substitution
! on a symmetric skyline global matrix.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::kv(:)
 real(dp),INTENT(IN OUT)::loads(0:)
 INTEGER,INTENT(IN)::kdiag(:)
 INTEGER::n,i,ki,l,m,j,it,k
 real(dp)::x
 n=UBOUND(kdiag,1)
 loads(1)=loads(1)/kv(1)
 DO i=2,n
   ki=kdiag(i)-i
   l=kdiag(i-1)-ki+1 
   x=loads(i)
   IF(l/=i)THEN
     m=i-1
     DO j=l,m 
       x=x-kv(ki+j)*loads(j)
     END DO
   END IF
   loads(i)=x/kv(ki+i)
 END DO
 DO it=2,n
   i=n+2-it
   ki=kdiag(i)-i
   x=loads(i)/kv(ki+i)
   loads(i)=x
   l=kdiag(i-1)-ki+1
   IF(l/=i)THEN
     m=i-1
     DO k=l,m
       loads(k)=loads(k)-x*kv(ki+k)
     END DO
   END IF
 END DO
 loads(1)=loads(1)/kv(1)
RETURN
END function spabac               
!
integer function checon(loads,oldlds,tol,converged, relerr)
!
! This function sets converged to .FALSE. if relative change in loads
! and oldlds is greater than tol and updates oldlds.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::loads(0:),tol
 real(dp),INTENT(IN OUT)::oldlds(0:)
 real(dp),INTENT(OUT)::relerr
 LOGICAL,INTENT(OUT)::converged
 CONVERGED=.TRUE.
 relerr = MAXVAL(ABS(loads-oldlds))/MAXVAL(ABS(loads))
 CONVERGED=(relerr<=tol)
 oldlds=loads
RETURN
END function checon

integer function invar(stress,sigm,dsbar,theta)
!
! This function forms the stress invariants in 2- or 3-d.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::stress(:)
 real(dp),INTENT(OUT),OPTIONAL::sigm,dsbar,theta
 real(dp)::sx,sy,sz,txy,dx,dy,dz,xj3,sine,s1,s2,s3,s4,s5,s6,ds1,ds2,ds3, &
   d2,d3,sq3,zero=0.0_dp,small=1.e-10,one=1.0_dp,two=2.0_dp,       &
   three=3.0_dp,six=6.0_dp,thpt5=13.5_dp
 INTEGER::nst 
 nst=UBOUND(stress,1)
 SELECT CASE(nst)
 CASE(4)
   sx=stress(1)
   sy=stress(2)
   txy=stress(3)
   sz=stress(4)
   sigm=(sx+sy+sz)/three
   dsbar=SQRT((sx-sy)**2+(sy-sz)**2+(sz-sx)**2+six*txy**2)/SQRT(two)
   IF(dsbar<small)THEN
     theta=zero
   ELSE
     dx=(two*sx-sy-sz)/three
     dy=(two*sy-sz-sx)/three
     dz=(two*sz-sx-sy)/three
     xj3=dx*dy*dz-dz*txy**2
     sine=-thpt5*xj3/dsbar**3
     IF(sine>=one)sine=one
     IF(sine<-one)sine=-one
     theta=ASIN(sine)/three
   END IF
 CASE(6)
   sq3=SQRT(three)
   s1=stress(1)  
   s2=stress(2)
   s3=stress(3) 
   s4=stress(4)
   s5=stress(5)
   s6=stress(6)
   sigm=(s1+s2+s3)/three
   d2=((s1-s2)**2+(s2-s3)**2+(s3-s1)**2)/six+s4*s4+s5*s5+s6*s6
   ds1=s1-sigm 
   ds2=s2-sigm  
   ds3=s3-sigm
   d3=ds1*ds2*ds3-ds1*s5*s5-ds2*s6*s6-ds3*s4*s4+two*s4*s5*s6
   dsbar=sq3*SQRT(d2)
   IF(dsbar<small)THEN
     theta=zero
   ELSE
     sine=-three*sq3*d3/(two*SQRT(d2)**3)
     IF(sine>=one)sine=one 
     IF(sine<-one)sine=-one 
     theta=ASIN(sine)/three
   END IF
 CASE DEFAULT
   WRITE(*,*)"wrong size for nst in invar"
 END SELECT
RETURN
END function invar

integer function mocouf(phi,c,sigm,dsbar,theta,f)
!
! This function calculates the value of the yield function
! for a Mohr-Coulomb material (phi in degrees).
!
 IMPLICIT NONE
 real(dp),INTENT(IN)::phi,c,sigm,dsbar,theta   
 real(dp),INTENT(OUT)::f
 real(dp)::csph,csth,snth,phir,snph 
 real(dp)::one=1.0_dp,d3=3.0_dp,d4=4.0_dp,d180=180.0_dp
 phir=phi*d4*ATAN(one)/d180
 snph=SIN(phir) 
 csph=COS(phir) 
 csth=COS(theta)
 snth=SIN(theta)
 f=snph*sigm+dsbar*(csth/SQRT(d3)-snth*snph/d3)-c*csph
 mocouf = 0
END function mocouf

integer function mocouq(psi,dsbar,theta,dq1,dq2,dq3)
!
! This function forms the derivatives of a Mohr-Coulomb potential
! function with respect to the three invariants (psi in degrees).
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::psi,dsbar,theta
 real(dp),INTENT(OUT)::dq1,dq2,dq3
 real(dp)::psir,snth,snps,sq3,c1,csth,cs3th,tn3th,tnth,zero=0.0_dp,     &
   pt49=0.49_dp,pt5=0.5_dp,one=1.0_dp,d3=3.0_dp,d4=4.0_dp,           &
   d180=180.0_dp
 psir=psi*d4*ATAN(one)/d180 
 snth=SIN(theta) 
 snps=SIN(psir)
 sq3=SQRT(d3)  
 dq1=snps
 if(ABS(snth).GT.pt49)THEN
   c1=one
   IF(snth.LT.zero)c1=-one
   dq2=(sq3*pt5-c1*snps*pt5/sq3)*sq3*pt5/dsbar 
   dq3=zero
 ELSE
   csth=COS(theta)
   cs3th=COS(d3*theta)
   tn3th=TAN(d3*theta)
   tnth=snth/csth
   dq2=sq3*csth/dsbar*((one+tnth*tn3th)+snps*(tn3th-tnth)/sq3)*pt5
   dq3=pt5*d3*(sq3*snth+snps*csth)/(cs3th*dsbar*dsbar)
 END IF
RETURN
END function mocouq
!
integer function formm(stress,m1,m2,m3)
!
! This function forms the derivatives of the invariants with respect to
! stress in 2- or 3-d.
!
 IMPLICIT NONE
 !INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 real(dp),INTENT(IN)::stress(:)
 real(dp),INTENT(OUT)::m1(:,:),m2(:,:),m3(:,:)
 real(dp)::sx,sy,txy,tyz,tzx,sz,dx,dy,dz,sigm,zero=0.0_dp,one=1.0_dp,  &
   two=2.0_dp,three=3.0_dp,six=6.0_dp,nine=9.0_dp
 INTEGER::nst,i,j
 nst=UBOUND(stress,1)
 SELECT CASE(nst)
 CASE(4)
   sx=stress(1)
   sy=stress(2)
   txy=stress(3)
   sz=stress(4)
   dx=(two*sx-sy-sz)/three
   dy=(two*sy-sz-sx)/three
   dz=(two*sz-sx-sy)/three
   sigm=(sx+sy+sz)/three
   m1=zero
   m2=zero
   m3=zero
   m1(1,1:2)=one
   m1(2,1:2)=one
   m1(4,1:2)=one
   m1(1,4)=one
   m1(4,4)=one
   m1(2,4)=one
   m1=m1/nine/sigm
   m2(1,1)=two/three
   m2(2,2)=two/three
   m2(4,4)= two/three
   m2(2,4)=-one/three
   m2(4,2)=-one/three
   m2(1,2)=-one/three
   m2(2,1)=-one/three
   m2(1,4)=-one/three
   m2(4,1)=-one/three
   m2(3,3)=two
   m3(3,3)=-dz
   m3(1:2,3)=txy/three
   m3(3,1:2)=txy/three
   m3(3,4)=-two*txy/three
   m3(4,3)=-two*txy/three
   m3(1,1)=dx/three
   m3(2,4)=dx/three
   m3(4,2)=dx/three
   m3(2,2)=dy/three
   m3(1,4)=dy/three
   m3(4,1)=dy/three
   m3(4,4)=dz/three
   m3(1,2)=dz/three
   m3(2,1)=dz/three
 CASE(6)
   sx=stress(1)
   sy=stress(2)    
   sz=stress(3)
   txy=stress(4)  
   tyz=stress(5) 
   tzx=stress(6)
   sigm=(sx+sy+sz)/three
   dx=sx-sigm  
   dy=sy-sigm 
   dz=sz-sigm
   m1=zero
   m2=zero
   m1(1:3,1:3)=one/(three*sigm)
   DO i=1,3 
     m2(i,i)=two 
     m2(i+3,i+3)=six 
   END DO
   m2(1,2)=-one
   m2(1,3)=-one 
   m2(2,3)=-one
   m3(1,1)=dx
   m3(1,2)=dz 
   m3(1,3)=dy 
   m3(1,4)=txy  
   m3(1,5)=-two*tyz
   m3(1,6)=tzx 
   m3(2,2)=dy 
   m3(2,3)=dx 
   m3(2,4)=txy
   m3(2,5)=tyz 
   m3(2,6)=-two*tzx 
   m3(3,3)=dz
   m3(3,4)=-two*txy
   m3(3,5)=tyz 
   m3(3,6)=tzx
   m3(4,4)=-three*dz 
   m3(4,5)=three*tzx
   m3(4,6)=three*tyz
   m3(5,5)=-three*dx
   m3(5,6)=three*txy 
   m3(6,6)=-three*dy
   DO i=1,6 
     DO j=i+1,6
       m1(j,i)=m1(i,j) 
       m2(j,i)=m2(i,j)   
       m3(j,i)=m3(i,j)
     END DO
   END DO
   m1=m1/three
   m2=m2/three
   m3=m3/three
 CASE DEFAULT
   WRITE(*,*)"nst size not recognised in formm"
 END SELECT
RETURN   
END function formm
END module main

integer function p64(resa, meshdims, meshnums, tola, srfa, nsrf, prop, &
   np_types, g_coord, nn, NodeDisp,g_num, nels)
!-------------------------------------------------------------------------
! Program 6.4 Plane strain slope stability analysis of an elastic-plastic
!             (Mohr-Coulomb) material using 8-node rectangular
!             quadrilaterals. Viscoplastic strain method.
!-------------------------------------------------------------------------
  use main
  implicit none 
 integer, intent(in):: meshnums(2,2), nsrf, np_types, nn, nels
 integer, intent(out):: g_num(8,nels)
 real(dp),INTENT(IN)::meshdims(3,2), tola(3), srfa(nsrf), prop(7,np_types)
 real(dp),INTENT(OUT)::resa(nsrf,5), g_coord(2,nn), NodeDisp(nsrf*3,nn)  
 
 
 
INTEGER::i,j,iel,iErr,iters,iy,limit,neq,nlen,ndim=2,ndof=16,nip=4
INTEGER::nod=8,nodof=2,nprops=7,nst=4,nx1,nx2,ny1,ny2 

 REAL(dp)::cf,ddt,det,dq1,dq2,dq3,dsbar,MaxDef,dt=1.0e15_dp,d4=4.0_dp, &
   d180=180.0_dp,e,f,fmax,h1,h2,lode_theta,one=1.0_dp,phi,phif,pi,psi,  &
   psif,sigm,snph,start_dt=1.e15_dp,s1,tnph,tnps,tol,two=2.0_dp,v,w1,w2,&
   zero=0.0_dp,STime, ETime, relerr
 CHARACTER(LEN=15)::argv,element='quadrilateral' 
 LOGICAL::converged       
 
 !-----------------------dynamic arrays------------------------------------
 INTEGER,ALLOCATABLE::etype(:),g(:),g_g(:,:),kdiag(:),nf(:,:), &
   num(:)  
 REAL(dp),ALLOCATABLE::bdylds(:),bee(:,:),bload(:), coord(:,:),dee(:,:),  &
   devp(:),elastic(:),eld(:),eload(:),eps(:),erate(:),evp(:),evpt(:,:,:), &
   flow(:,:),fun(:),gravlo(:),km(:,:),kv(:),loads(:),m1(:,:),&
   m2(:,:),m3(:,:),oldis(:),points(:,:),sigma(:),srf(:),        &
   weights(:)  
!-----------------------input and initialisation--------------------------
 
 w1 = meshdims(1,1)
 s1 = meshdims(2,1)
 w2 = meshdims(3,1)
 h1 = meshdims(1,2)
 h2 = meshdims(2,2)
 nx1 = meshnums(1,1)
 nx2 = meshnums(2,1)
 ny1 = meshnums(1,2)
 ny2 = meshnums(2,2)
 
ALLOCATE(nf(nodof,nn),points(nip,ndim),weights(nip),    &
   num(nod),dee(nst,nst),evpt(nst,nip,nels),coord(nod,ndim),fun(nod),     &
   g_g(ndof,nels),bee(nst,ndof),km(ndof,ndof),eld(ndof),  &
   eps(nst),sigma(nst),bload(ndof),eload(ndof),erate(nst),evp(nst),       &
   devp(nst),g(ndof),m1(nst,nst),m2(nst,nst),m3(nst,nst),flow(nst,nst),   &
   etype(nels))  !prop(nprops,np_types),g_coord(ndim,nn),g_num(nod,nels),
!
 
  etype(1:int(prop(7,1))) = 1  
  do i=2,np_types
      etype(int(prop(7,i-1))+1:int(prop(7,i))) = i
  end do
 
 iErr =  emb_2d_bc(nx1,nx2,ny1,ny2,nf) 
 
 neq= MAXVAL(nf)

 ALLOCATE(kdiag(neq),loads(0:neq),bdylds(0:neq),oldis(0:neq),             &
   gravlo(0:neq),elastic(0:neq)) 
 !ALLOCATE(NodeDisp(nn,3))
   kdiag=0
!
!-----------------------loop the elements to find global arrays sizes-----
 CALL cpu_time(STime)
 elements_1: DO iel=1,nels
   iErr =  emb_2d_geom(iel,nx1,nx2,ny1,ny2,w1,s1,w2,h1,h2,coord,num)
   iErr =  num_to_g(num,nf,g) 
   g_num(:,iel)=num
   g_coord(:,num)=TRANSPOSE(coord) 
   g_g(:,iel)=g 
   iErr =  fkdiag(kdiag,g)
 END DO elements_1

 DO i=2,neq 
   kdiag(i)=kdiag(i)+kdiag(i-1) 
 END DO 
 ALLOCATE(kv(kdiag(neq)))

 

iErr =  sample(element,points,weights) 
 
 kv=zero 
 gravlo=zero 

!-----------------------element stiffness integration and assembly--------
 elements_2: DO iel=1,nels
   iErr =  deemat(dee,prop(5,etype(iel)),prop(6,etype(iel))) 
   num=g_num(:,iel)
   coord=TRANSPOSE(g_coord(:,num)) 
   g=g_g(:,iel) 
   km=zero 
   eld=zero
   gauss_pts_1: DO i=1,nip
     iErr =  shape_fun(fun,points,i)
     iErr =  bee8(bee,coord,points(i,1),points(i,2),det)
     km=km+MATMUL(MATMUL(TRANSPOSE(bee),dee),bee)*det*weights(i)
     eld(nodof:ndof:nodof)=eld(nodof:ndof:nodof)+fun(:)*det*weights(i)
   END DO gauss_pts_1
   !if (iel==200) then

   !  kmout = km
   !end if
   iErr =  fsparv(kv,km,g,kdiag) 
   gravlo(g)=gravlo(g)-eld*prop(4,etype(iel))
   
 END DO elements_2
 
!-----------------------factorise equations-------------------------------
 iErr =  sparin(kv,kdiag) 
 pi=ACOS(-one)
 
 !
!-----------------------trial strength reduction factor loop--------------
! READ(10,*)tol,limit,nsrf 
 tol = tola(1)
 limit = int(tola(2))
! nsrf = int(tola(3))
! ALLOCATE(srf(nsrf)) 
 !READ(10,*)srf
 ! srf = indat(:,5+np_types)
! WRITE(11,'(/A)')"    srf    max disp  iters"
 srf_trials: DO iy=1,nsrf
   dt=start_dt
   DO i=1,np_types
     phi=prop(1,i) 
     tnph=TAN(phi*pi/d180) 
     phif=ATAN(tnph/srfa(iy)) 
     snph=SIN(phif) 
     e=prop(5,i) 
     v=prop(6,i)
     ddt=d4*(one+v)*(one-two*v)/(e*(one-two*v+snph**2)) 
     IF(ddt<dt)dt=ddt
   END DO 
   iters=0 
   bdylds=zero 
   evpt=zero 
   oldis=zero

!-----------------------plastic iteration loop----------------------------
   its: DO
     fmax=zero 
     iters=iters+1 
     loads=gravlo+bdylds 
     iErr =  spabac(kv,loads,kdiag) 
     loads(0)=zero
     
     IF(iy==1.AND.iters==1)elastic=loads
!-----------------------check plastic convergence-------------------------
     iErr =  checon(loads,oldis,tol,converged,relerr)
     IF(iters==1)converged=.FALSE.
     IF(converged.OR.iters==limit)bdylds=zero
!-----------------------go round the Gauss Points ------------------------
     elements_3: DO iel=1,nels
       bload=zero 
       phi=prop(1,etype(iel)) 
       tnph=TAN(phi*pi/d180)
       phif=ATAN(tnph/srfa(iy))*d180/pi 
       psi=prop(3,etype(iel))
       tnps=TAN(psi*pi/d180) 
       psif=ATAN(tnps/srfa(iy))*d180/pi
       cf=prop(2,etype(iel))/srfa(iy) 
       e=prop(5,etype(iel)) 
       v=prop(6,etype(iel)) 
       iErr =  deemat(dee,e,v) 
       num=g_num(:,iel)
       coord=TRANSPOSE(g_coord(:,num)) 
       g=g_g(:,iel) 
       eld=loads(g)
       gauss_pts_2: DO i=1,nip
         iErr =  bee8(bee,coord,points(i,1),points(i,2),det)
         eps=MATMUL(bee,eld) 
         eps=eps-evpt(:,i,iel) 
         sigma=MATMUL(dee,eps)
         iErr =  invar(sigma,sigm,dsbar,lode_theta)
!-----------------------check whether yield is violated-------------------
         iErr =  mocouf(phif,cf,sigm,dsbar,lode_theta,f) 
         IF(f>fmax)fmax=f
         IF(converged.OR.iters==limit)THEN 
           devp=sigma 
         ELSE
           IF(f>=zero.OR.(converged.OR.iters==limit))THEN
             iErr =  mocouq(psif,dsbar,lode_theta,dq1,dq2,dq3)
             iErr =  formm(sigma,m1,m2,m3) 
             flow=f*(m1*dq1+m2*dq2+m3*dq3)
             erate=MATMUL(flow,sigma) 
             evp=erate*dt
             evpt(:,i,iel)=evpt(:,i,iel)+evp 
             devp=MATMUL(dee,evp)
           END IF
         END IF
         IF(f>=zero)THEN
           eload=MATMUL(devp,bee) 
           bload=bload+eload*det*weights(i)
         END IF
       END DO gauss_pts_2
!-----------------------compute the total bodyloads vector----------------
       bdylds(g)=bdylds(g)+bload 
       bdylds(0)=zero
     END DO elements_3

     IF(converged.OR.iters==limit)EXIT
   END DO its
    call cpu_time(ETime)

    MaxDef = zero
    DO i = 1,nn
        NodeDisp((iy-1)*3+1,i) = loads(nf(1, i))
        NodeDisp((iy-1)*3+2,i) = loads(nf(2, i))
        NodeDisp((iy-1)*3+3,i) = Sqrt((loads(nf(1, i)) - elastic(nf(1, i))) ** 2 + (loads(nf(2, i)) - elastic(nf(2, i))) ** 2)
        !If (Abs(NodeDisp(3,i)) > MaxDef) MaxDef = Abs(NodeDisp(3,i))
    END DO

!   IF(iters==limit)EXIT
  resa(iy,1) = srfa(iy)
  resa(iy,2) = iters
  resa(iy,3) = relerr
  resa(iy,5) = maxval(abs(loads))
  resa(iy,4) = etime-stime
END DO srf_trials 

 
 p64 = prop(7,1)
END function p64

integer function getbee(bee,coord,xi,eta,det)
  use main
  implicit none
 
  real(dp),INTENT(IN)::coord(8,2),xi,eta
  real(dp),INTENT(OUT)::bee(4,16),det
  integer :: iErr
  
  iErr = bee8(bee,coord,xi,eta,det)
  getbee = iErr
end function getbee


