from __future__ import division

import scipy.linalg as sla
import scipy.sparse as ssp
import scipy.sparse.linalg as sspla
import numpy  as np
from ctypes import byref, cdll, c_int, c_double, c_char_p
import numpy.ctypeslib as npc
import os
import time
import xlwings as xw

#Load p64s.dll
try:
    wkbpath = os.path.dirname(os.path.abspath(__file__))
    p64dll = cdll.LoadLibrary(wkbpath + '/p64s.dll')
except:
    p64dll = cdll.LoadLibrary('./p64s.dll')
    
def py64(dataa, tola, srfa, propa):
# Convert dataa to Numpy array, and split into double and integer arrays
    dataa = np.array(dataa)
    meshdims = dataa[0:2,:]
    meshnums = dataa[2:4,0:2].astype(int)

#Calculate size of output arrays    
    np_types = int(dataa[4,0])
    nx1 = meshnums[0,0]
    nx2 = meshnums[0,1]
    ny1 = meshnums[1,0]
    ny2 = meshnums[1,1]
    nye=ny1+ny2
    nels=nx1*nye+ny2*nx2
    nn=(3*nye+2)*nx1+2*nye+1+(3*ny2+2)*nx2
#Create Numpy arrays and convert to ctypes pointers
#Convert integers to c_int
    coorda = np.zeros((nn,2))
    coordp = npc.as_ctypes(coorda)
    g_num = np.zeros((nels,8), dtype=np.int)
    g_nump = npc.as_ctypes(g_num)
    srfa = np.array(srfa)[:,0]
    nsrfi = srfa.shape[0]
    nsrf = c_int(nsrfi)
    nodedisa = np.zeros((nn,nsrfi*3))
    nodedisp = npc.as_ctypes(nodedisa)
    nnp = c_int(nn)
    meshdimp = npc.as_ctypes(meshdims)
    meshnump = npc.as_ctypes(meshnums)
    tolp = npc.as_ctypes(np.array(tola)[0])
    srfp = npc.as_ctypes(srfa)
    propa = np.array(propa)
    propp = npc.as_ctypes(propa)
    resa = np.zeros((5,nsrfi))
    resp = npc.as_ctypes(resa)
    np_types = c_int(np_types)
    nelsp = c_int(nels)
    
 #Call p64 with all ctype arguments passed by reference   
    iErr = p64dll.p64(byref(resp), byref(meshdimp), byref(meshnump), byref(tolp), byref(srfp), byref(nsrf), 
    byref(propp), byref(np_types), byref(coordp), byref(nnp), byref(nodedisp), byref(g_nump), byref(nelsp))
 
 #Return results using original Python variables
    return [resa, coorda, nodedisa, g_num, nn,  nels] 

def getf(phi,c,sigm,dsbar,theta):
    p64dll = cdll.LoadLibrary('./maindll.dll')
    phi = c_double(phi)
    c = c_double(c)
    sigm = c_double(sigm)
    dsbar = c_double(dsbar)
    theta = c_double(theta)
    f = c_double(0.0)
    phir = c_double(0.0)
    snph = c_double(0.0)
    frtn = p64dll.mocouf(byref(phi),byref(c),byref(sigm),byref(dsbar),byref(theta),byref(f))
    return np.float64(f)
    
    
def getbee(coord,xi,eta):

    bee = np.zeros((16,4))
    beep = npc.as_ctypes(bee)
    coord = npc.as_ctypes(np.array(coord))
    xi= c_double(xi)
    eta = c_double(eta)
    det = c_double(0.0)

    ierr = p64dll.getbee(byref(beep),byref(coord),byref(xi),byref(eta),byref(det))
    return [npc.as_array(beep).tolist(),np.float64(ierr)]
    
def cwd():
    return [os.getcwd(), os.path.realpath(__file__), os.path.dirname(os.path.abspath(__file__))] 
